//Created by Amin Rahmani, Ph.D., P.E. - August 12, 2017
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file gets input values from user, reads input CPT file (in text format), calculates Ic per Robertson and Wride (1998), and calculates end bearing capacity and skin friction per Bustamante and Gianeselli (1982).
// The analysis is performed based on CPT data which must be in Imperial units : depth (ft), cone resistance and sleeve friction in (tsf) and water pressure in (psi). The data must start from line 1 (no header).
// The code is developed for pile groups I_A,B and II_A,B.
//===========================================================================================================================

#include "GeoCapac_B&G_1982.hpp"

//Default constructor function
B_G_1982::B_G_1982 () {
    numDataPoint=0;
    a_corr = 0.8;
    count =0;
    sum = 0.0;
    count_f =0;
    sum_f = 0.0;
    sum_Ic = 0.0;
    Qp_ult = 0.0;
    fs_ult = 0.0;
}

//Destructor function
B_G_1982::~B_G_1982 () {
    //Do nothing
}

//This function reads the input CPTdata
void B_G_1982::fileReader (string inputcptData) {
    
    this -> inputcptData = inputcptData;
    ifstream inputData;
    inputData.open (inputcptData);
    
    double col1, col2, col3, col4;
    
    if ( inputData.fail()) {
        cerr << "Error in opening the input file" << endl;
        exit(1);
    } else {
        while (! inputData.eof() ) {
            inputData >> col1 >> col2  >> col3 >> col4;
            depth.push_back(col1);
            q_c.push_back(col2);
            f_s.push_back(col3);
            u_water.push_back(col4);
            numDataPoint++;
        }
        inputData.close();
    }
    
}

//This function gets and sets the value of user defined member variables
void B_G_1982::getValues (double GWdepth, double columnDiam, double columnDepth, double zeroFricDepth, double FoS_end, double FoS_skin, string groupID_end, string groupID_skin) {
    
    this -> GWdepth = GWdepth;
    this -> columnDiam = columnDiam;
    this -> columnDepth = columnDepth;
    this -> zeroFricDepth = zeroFricDepth;
    this -> FoS_end = FoS_end;
    this -> FoS_skin = FoS_skin;
    this -> groupID_end = groupID_end;
    this -> groupID_skin = groupID_skin;
    
}

// Calculates Ic per Robertson & Wride (1998) - Ic is required to classify soil type (silt, clay, sand)
void B_G_1982::calc_Ic () {
    
    for (int i=0; i < numDataPoint; i++) {
        
        //Corrected cone resistance
        q_t.push_back(q_c[i]+(u_water[i]*144/2000)*(1-a_corr));
        
        //soil unit weight
        gamma_t.push_back((0.27*log10(f_s[i]/q_t[i]*100)+0.36*log10(q_t[i]*2000.0/2089)+1.236)*62.4);
        
        //Insitu stresses
        TotStress.push_back(gamma_t[i]*depth[i]/2000.0);
        
        if (depth[i] <= GWdepth) {
            insitu_hydr.push_back(0.0);
        } else {
            insitu_hydr.push_back(62.4*(depth[i]-GWdepth)/2000.0);
        }
        
        insitu_EffStress.push_back(TotStress[i]-insitu_hydr[i]);
        
        //Calculate Ic
        n_pow.push_back(1.0);
        double new_n_pow =0.5;
        
        CN.push_back(min(2.0,pow(1.058/insitu_EffStress[i],n_pow[i])));
        Qn.push_back((q_t[i]-TotStress[i])*CN[i]/1.058);
        Fn.push_back(f_s[i]*100.0/(q_t[i]-TotStress[i]));
        Ic.push_back(sqrt(pow((3.47-log10 (Qn[i])),2)+pow((1.22+log10 (Fn[i])),2)));
        
        // iteration to calculate Ic
        while ( abs(n_pow[i]-new_n_pow) > 0.01) {
            
            replace( CN.begin(), CN.end(), CN[i], min(2.0,pow(1.058/insitu_EffStress[i],n_pow[i])) );
            replace( Qn.begin(), Qn.end(), Qn[i], (q_t[i]-TotStress[i])*CN[i]/1.058 );
            replace( Ic.begin(), Ic.end(), Ic[i], sqrt(pow((3.47-log10 (Qn[i])),2)+pow((1.22+log10 (Fn[i])),2)) );
            
            new_n_pow=n_pow[i];
            
            //Iteration per Robertson & Wride (1998)
            if (Ic[i] <= 1.64) {
                replace( n_pow.begin(), n_pow.end(), n_pow[i], 0.5);
            } else if (Ic[i] <= 3.3) {
                replace( n_pow.begin(), n_pow.end(), n_pow[i], (Ic[i]-1.64)*0.3+0.5);
            } else {
                replace( n_pow.begin(), n_pow.end(), n_pow[i], 1.0);
            }
            
        }
    }
    
}

// This function calculates geotechnical capacity based on Bustamante and Gianeselli (1982)
void B_G_1982::calcGeoCap() {
    
    // Calculate qca
    //Step 1
    for (int i=0; i < numDataPoint; i++) {

        if ( depth[i] >= (columnDepth-1.5*columnDiam) && depth[i] <= (columnDepth+1.5*columnDiam)) {
            sum += q_t[i];
            sum_Ic += Ic[i];
            count++;
        }
    }

    qca1 = sum/count;
    Ica = sum_Ic/count;
    
    //Step 2 and 3
    for (int i=0; i < numDataPoint; i++) {
        
       if ( depth[i] >= (columnDepth-1.5*columnDiam) && depth[i] <= (columnDepth+1.5*columnDiam) && q_t[i] < 1.3*qca1 && q_t[i] > 0.7*qca1) {
                sum_f += q_t[i];
                count_f++;
       }
    }
    qca = sum_f/count_f;
    
    //Calculate ultimate end bearing capacity in tons   
    if ( groupID_end == "I") {
            if (Ica > 2.6 && qca < 1.0*10.44) {
                kc = 0.4;
            } else if ( Ica > 2.6 && qca < 5.0*10.44 ) {
                kc=0.35;
            } else if ( Ica > 2.05 && qca < 5.0*10.44 ) {
                kc=0.45;
            } else if ( Ica < 2.05 && qca < 12.0*10.44 ) {
                kc=0.40;
            } else {
                kc=0.30;
            }
    } else {
        if (Ica > 2.6 && qca < 1.0*10.44) {
            kc = 0.5;
        } else if ( Ica > 2.6 && qca < 5.0*10.44 ) {
            kc=0.45;
        } else if ( Ica > 2.05 && qca < 5.0*10.44 ) {
            kc=0.55;
        } else if ( Ica < 2.05 && qca < 12.0*10.44 ) {
            kc=0.50;
        } else {
            kc=0.40;
        }
        
    }   
    Qp_ult = kc*qca*0.25*3.1416*pow(columnDiam,2);
    
    //Calculate ultimate skin friction in tons
    int i=0;
    fs_ult=0.0;
    
    while ( depth[i] < columnDepth ) {
        
        // if group IA and IIA is selected
        if ( groupID_skin == "IA" || groupID_skin == "IIA") {
            
            if (Ic[i] > 2.6 && q_t[i] < 1.0*10.44) {

                alpha = 30.0;
                fp = 0.015*10.44;
            } else if ( Ic[i] > 2.6 && q_t[i] < 5.0*10.44 ) {
                
                alpha = 40.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] > 2.05 && q_t[i] <= 5.0*10.44 ) {
                
                alpha = 60.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] > 2.6 && q_t[i] > 5.0*10.44 ) {
                
                alpha = 60.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] < 2.05 && q_t[i] < 12.0*10.44 ) {
                
                alpha = 100.0;
                fp = 0.08*10.44;
            } else {
                
                alpha = 150.0;
                fp = 0.12*10.44;
            }
			
         // if group IB is selected   
        } else if (groupID_skin == "IB") {
        
            if (Ic[i] > 2.6 && q_t[i] < 1.0*10.44) {
                
                alpha = 3000.0;
                fp = 0.015*10.44;
            } else if ( Ic[i] > 2.6 && q_t[i] < 5.0*10.44 ) {
                
                alpha = 80.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] > 2.05 && q_t[i] <= 5.0*10.44 ) {
                
                alpha = 150.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] > 2.6 && q_t[i] > 5.0*10.44 ) {
                
                alpha = 120.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] < 2.05 && q_t[i] < 12.0*10.44 ) {
                
                alpha = 200.0;
                fp = 0.035*10.44;
            } else {
                
                alpha = 300.0;
                fp = 0.08*10.44;
            }
			
         // if group IIB is selected    
        } else if (groupID_skin == "IIB") {
            
            if (Ic[i] > 2.6 && q_t[i] < 1.0*10.44) {
                
                alpha = 30.0;
                fp = 0.015*10.44;
            } else if ( Ic[i] > 2.6 && q_t[i] < 5.0*10.44 ) {
                
                alpha = 80.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] > 2.05 && q_t[i] <= 5.0*10.44 ) {
                
                alpha = 120.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] > 2.6 && q_t[i] > 5.0*10.44 ) {
                
                alpha = 120.0;
                fp = 0.035*10.44;
            } else if ( Ic[i] < 2.05 && q_t[i] < 12.0*10.44 ) {
                
                alpha = 200.0;
                fp = 0.08*10.44;
            } else {
                
                alpha = 200.0;
                fp = 0.12*10.44;
            }
            
        }
        
        skin_fric.push_back(min(q_t[i]/alpha,fp));
        
        if (i > 1 && depth[i] >= zeroFricDepth ) {
            
           fs_ult += ((skin_fric[i]+skin_fric[i-1])/2.0)*3.1416*columnDiam*(depth[i]-depth[i-1]);
        }
        
        i++;

    }
    
    // Print results
    cout << "Allowable End Bearing Capacity : " << int(Qp_ult/FoS_end) << " tons" << endl;
    cout << "Allowable Skin Friction : " << int(fs_ult/FoS_skin) << " tons" << endl;
    cout << "Column Total Geotechnical Capacity : " << int(Qp_ult/FoS_end+fs_ult/FoS_skin) << " tons" << endl;

}



